@@ -44,6 +44,11 @@ import socket
 import tempfile
 import codecs
 
+try:
+    _trace_memo_entries = [ int(i) for i in os.environ["SPICKLE_TRACE_MEMO_ENTRIES"].split() ]
+except Exception:
+    _trace_memo_entries = None
+
 SOCKET_PAIR_TYPE = None
 if hasattr(socket, "socketpair"):
     _sp = socket.socketpair()
@@ -373,19 +378,25 @@ class Pickler(pickle.Pickler):
         self.dispatch[types.FileType] = self.saveFile.__func__
         self.dispatch[socket.SocketType] = self.saveSocket.__func__
         self.dispatch[SOCKET_PAIR_TYPE] = self.saveSocketPairSocket.__func__
-        self.dispatch[WRAPPER_DESCRIPTOR_TYPE] = self.saveWrapperOrMethodDescriptor.__func__
-        self.dispatch[METHOD_DESCRIPTOR_TYPE] = self.saveWrapperOrMethodDescriptor.__func__
+        self.dispatch[WRAPPER_DESCRIPTOR_TYPE] = self.saveDescriptorWithObjclass.__func__
+        self.dispatch[METHOD_DESCRIPTOR_TYPE] = self.saveDescriptorWithObjclass.__func__
+        self.dispatch[types.MemberDescriptorType] = self.saveDescriptorWithObjclass.__func__
+        self.dispatch[types.GetSetDescriptorType] = self.saveDescriptorWithObjclass.__func__
         self.dispatch[staticmethod] = self.saveStaticOrClassmethod.__func__
         self.dispatch[classmethod] = self.saveStaticOrClassmethod.__func__
         self.dispatch[property] = self.saveProperty.__func__
         self.dispatch[operator.itemgetter] = self.saveOperatorItemgetter.__func__
         self.dispatch[operator.attrgetter] = self.saveOperatorAttrgetter.__func__
-        
+        self.dispatch[types.DictProxyType] = self.saveDictProxy.__func__
+
         # initiallize the module_dict_ids module dict lookup table
         # this stackless specific call creates the dict self.module_dict_ids
         self._pickle_moduledict(self, {})
         self.object_dict_ids = {}
         
+        # Used for class creation
+        self.delayedClassSetAttrList = []
+        
     def mustSerialize(self, obj):
         """test, if a module must be serialised"""
         # Legacy check for flowGuide2 workflow modules
@@ -470,6 +481,8 @@ class Pickler(pickle.Pickler):
 
     def save(self, obj):
         # Check for persistent id (defined by a subclass)
+        # set a magical variable for debugging
+        __object_to_be_pickled__ = obj
         pid = self.persistent_id(obj)
         if pid:
             self.save_pers(pid)
@@ -531,7 +544,7 @@ class Pickler(pickle.Pickler):
         if obj is EMPTY_SET_ITERATOR:
             return self.save_reduce(iter, (set(),), obj=obj)
                                 
-        # handle __new__ anbd similar methods of built-in types        
+        # handle __new__ and similar methods of built-in types        
         if (isinstance(obj, type(object.__new__)) and 
             getattr(obj,"__name__", None) in ('__new__', '__subclasshook__')):
             return self.saveBuiltinNew(obj)
@@ -564,10 +577,19 @@ class Pickler(pickle.Pickler):
 
         if id(obj) in self.memo:
             m = self.memo[id(obj)]
-            LOGGER().error("Object already in memo! Id: %d, Obj: %r of type %r, Memo: %d=%r of type %r", 
+            LOGGER().error("Object already in memo! Id: %d, Obj: %r of type %r, Memo-key: %d=%r of type %r", 
                          id(obj), obj, type(obj), 
                          m[0], m[1], type(m[1]))
-        return self.__class__.__bases__[0].memoize(self, obj)
+            self._dumpSaveStack()
+        self.__class__.__bases__[0].memoize(self, obj)
+        if _trace_memo_entries:
+            i = id(obj)
+            m = self.memo.get(i)
+            if isinstance(m, tuple) and m[0] in _trace_memo_entries:
+                LOGGER().error("Traced object added to memo! Id: %d, Obj: %r of type %r, Memo-key: %d=%r of type %r", 
+                         id(obj), obj, type(obj), 
+                         m[0], m[1], type(m[1]))
+                self._dumpSaveStack()
         
     def save_dict(self, obj):
         if obj is sys.modules:
@@ -699,7 +721,22 @@ class Pickler(pickle.Pickler):
         return self.save_reduce(obj=obj, *rv)
 
     def saveClass(self, obj):
+        # create a class in 2 steps
+        # 1. recursively create the class with all bases, but without any attributes
+        #    Queue all attribute settings for later execution
+        # 2. Set all attributes for all classes
+        #
+        # Why? Because a base-class might reference its child via the global 
+        # namespace of its methods. And then we might get an assertion error 
+        # form self.memoize
+        #
         write = self.write
+        dcsal = self.delayedClassSetAttrList
+        isFirstClassCreation = not dcsal # test if list is empty
+        if isFirstClassCreation:
+            # a class creation is in progress
+            dcsal.append(True)
+        
         f = type(obj)
         name = obj.__name__
         d1 = {}; d2 = {}
@@ -707,13 +744,15 @@ class Pickler(pickle.Pickler):
             if k in ('__doc__', '__module__', '__slots__'):
                 d1[k] = v
                 continue
-            if type(v) in (types.DictProxyType, types.GetSetDescriptorType, types.MemberDescriptorType):
-                continue
             if k in ('__dict__', '__class__' ):
                 continue
+            if type(v) in (types.GetSetDescriptorType, types.MemberDescriptorType):
+                if v.__name__ == k and v.__objclass__ is obj:
+                    continue
             d2[k] = v
+
         self.save_reduce(f,(name, obj.__bases__, d1), obj=obj)
-        
+
         # Use setattr to set the remaining class members. 
         # unfortunately, we can't use the state parameter of the 
         # save_reduce method (pickle BUILD-opcode), because BUILD 
@@ -723,8 +762,17 @@ class Pickler(pickle.Pickler):
         keys.sort()
         for k in keys:
             v = d2[k]
-            self.save_reduce(setattr, (obj, k, v))
-            write(pickle.POP)
+            dcsal.append((obj, k, v))
+
+        if isFirstClassCreation:
+            operations = dcsal[1:] # skip the True
+            del dcsal[:] # clear the list to enable recursions
+            
+            # perform the attribute assignments in the same order as they were recorded
+            for t in operations:
+                self.save_reduce(setattr, t)
+                write(pickle.POP)
+
 
     def saveModule(self, obj, reload=False):
         write = self.write
@@ -823,7 +871,7 @@ class Pickler(pickle.Pickler):
                         (obj, objSelf, objName))
         return self.save_reduce(getattr, (objSelf, objName), obj=obj)
         
-    def saveWrapperOrMethodDescriptor(self, obj):
+    def saveDescriptorWithObjclass(self, obj):
         t = obj.__objclass__
         if obj is not getattr(t, obj.__name__):
             raise pickle.PicklingError("Can't pickle %r: it's not the same object as %s.%s" %
@@ -868,6 +916,48 @@ class Pickler(pickle.Pickler):
         obj(probe)
         return self.save_reduce(type(obj), tuple(record), obj=obj)
     
+    def saveDictProxy(self, obj):
+        attr = obj.get("__dict__")
+        if isinstance(attr, types.GetSetDescriptorType) and attr.__name__ == "__dict__" and getattr(attr.__objclass__, "__dict__", None) == obj:
+            # probably a dict proxy of class attr.__objclass__
+            return self.save_reduce(getattr, (attr.__objclass__, '__dict__'), obj=obj)
+        raise pickle.PicklingError("Can't pickle %r: it is not the dict-proxy of a class dictionary" % (obj,))
+    
+    def _dumpSaveStack(self):
+        from inspect import currentframe
+        from pprint import pformat
+        fr = currentframe()
+        obj = None
+        try:
+            while fr is not None:
+                try:
+                    obj = fr.f_locals['__object_to_be_pickled__']
+                except Exception:
+                    pass
+                else:
+                    try:
+                        s = pformat(obj, depth=3)
+                    except Exception:
+                        try:
+                            s = repr(obj)
+                        except Exception:
+                            s = "<NO REPRESENTATION AVAILABLE>"
+                    try:
+                        t = str(type(obj))
+                    except Exception:
+                        t = "<NO REPRESENTATION AVAILABLE>"
+                        
+                    i = id(obj)
+                    try:
+                        m = self.memo[i][0]
+                    except Exception:
+                        m = "n.a."
+                        
+                    LOGGER().info("Thing to be pickled id=%d, memo-key=%s, type=%s: %s" % (i, m, t, s))
+                fr = fr.f_back
+        finally:
+            del fr
+
         
 class SPickleTools(object):
     """A collection of simple utility methods.
